@@ -213,10 +213,6 @@ class Balloon:
         if len(self.window) > side:
             self.window[side].set_balloon(num)
 
-    def set_direction(self, side, dir):
-        if len(self.window) > side:
-            self.window[side].set_direction(dir)
-
     def set_position(self, side, base_x, base_y):
         if len(self.window) > side:
             self.window[side].set_position(base_x, base_y)
@@ -399,20 +395,28 @@ class BalloonWindow:
         self.text_count = 0
         self.balloon_surface = None
         self.autoscroll = True
+        self.dragged = False
+        self.x_root = None
+        self.y_root = None
+        self.x_fractions = 0
+        self.y_fractions = 0
         # create drawing area
         self.darea = Gtk.DrawingArea()
         self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK|
                               Gdk.EventMask.BUTTON_PRESS_MASK|
+                              Gdk.EventMask.BUTTON_RELEASE_MASK|
                               Gdk.EventMask.POINTER_MOTION_MASK|
                               Gdk.EventMask.POINTER_MOTION_HINT_MASK|
                               Gdk.EventMask.SCROLL_MASK)
         self.darea.connect('draw', self.redraw)
         self.darea.connect('button_press_event', self.button_press)
+        self.darea.connect('button_release_event', self.button_release)
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('scroll_event', self.scroll)
         self.window.add(self.darea)
         self.window.realize()
-        self.window.get_window().set_background_pattern(None) # XXX
+        self.window.override_background_color(Gtk.StateFlags.NORMAL,
+                                              Gdk.RGBA(0, 0, 0, 0))
         self.layout = Pango.Layout(self.darea.get_pango_context())
         self.sstp_layout = Pango.Layout(self.darea.get_pango_context())
         mask_r = desc.get_with_type('maskcolor.r', int, 128)
@@ -433,7 +437,7 @@ class BalloonWindow:
         sstp_b = desc.get_with_type('sstpmessage.font.color.b', int, text_b)
         self.sstp_message_color = (sstp_r / 255., sstp_g / 255., sstp_b / 255.)
         # initialize
-        self.direction = min(side, 1) ## kluge: multi character
+        self.__direction = min(side, 1) ## kluge: multi character
         self.position = (0, 0)
         self.reset_fonts()
         self.clear_text()
@@ -443,9 +447,19 @@ class BalloonWindow:
 
     @property
     def scale(self):
-        scalling = self.request_parent('GET', 'get_preference', 'balloon_scalling')
+        scaling = self.request_parent('GET', 'get_preference', 'balloon_scaling')
         scale = self.request_parent('GET', 'get_preference', 'surface_scale')
-        return scale if scalling else 100 # [%]
+        return scale if scaling else 100 # [%]
+
+    @property
+    def direction(self):
+        return self.__direction
+
+    @direction.setter
+    def direction(self, direction):
+        if self.__direction != direction:
+            self.__direction = direction # 0: left, 1: right
+            self.reset_balloon()
 
     def get_balloon_windowposition(self):
         winpos_x = self.__get_with_scaling('windowposition.x', int, 0)
@@ -626,7 +640,10 @@ class BalloonWindow:
         x, y = self.position
         self.width = self.balloon_surface.get_width()
         self.height = self.balloon_surface.get_height()
-        self.darea.set_size_request(self.width, self.height)
+        scale = self.scale
+        w = int(self.width * scale / 100.0)
+        h = int(self.height * scale / 100.0)
+        self.darea.set_size_request(w, h)
         self.reset_arrow()
         self.reset_sstp_marker()
         self.reset_message_regions()
@@ -634,11 +651,6 @@ class BalloonWindow:
         if self.__shown:
             self.darea.queue_draw()
 
-    def set_direction(self, dir):
-        if self.direction != dir:
-            self.direction = dir # 0: left, 1: right
-            self.reset_balloon()
-
     def set_autoscroll(self, flag):
         self.autoscroll = bool(flag)
 
@@ -1012,15 +1024,28 @@ class BalloonWindow:
             _, x, y, state = widget.get_window().get_pointer()
         else:
             x, y, state = event.x, event.y, event.get_state()
-        if not self.link_buffer:
-            return True
-        px = int(x)
-        py = int(y)
         scale = self.scale
-        px = int(px * 100 / scale)
-        py = int(py * 100 / scale)
-        if self.check_link_region(px, py):
-            widget.queue_draw()
+        if self.link_buffer:
+            px = int(x)
+            py = int(y)
+            px = int(px * 100 / scale)
+            py = int(py * 100 / scale)
+            if self.check_link_region(px, py):
+                widget.queue_draw()
+        if not self.request_parent('GET', 'busy'):
+            if state & Gdk.ModifierType.BUTTON1_MASK:
+                if self.x_root is not None and \
+                   self.y_root is not None:
+                    self.dragged = True
+                    x_delta = (event.x_root - self.x_root) * 100 / scale + self.x_fractions
+                    y_delta = (event.y_root - self.y_root) * 100 / scale + self.y_fractions
+                    self.x_fractions = x_delta - int(x_delta)
+                    self.y_fractions = y_delta - int(y_delta)
+                    self.request_parent(
+                        'NOTIFY', 'update_balloon_offset',
+                        self.side, int(x_delta), int(y_delta))
+                    self.x_root = event.x_root
+                    self.y_root = event.y_root
         return True
 
     def scroll(self, darea, event):
@@ -1084,6 +1109,22 @@ class BalloonWindow:
         # balloon's background
         self.request_parent('NOTIFY', 'notify_balloon_click',
                             event.button, click, self.side)
+        self.x_root = event.x_root
+        self.y_root = event.y_root
+        return True
+
+    def button_release(self, window, event):
+        x = int(event.x)
+        y = int(event.y)
+        scale = self.scale
+        x = int(x * 100 / scale)
+        y = int(y * 100 / scale)
+        if self.dragged:
+            self.dragged = False
+        self.x_root = None
+        self.y_root = None
+        self.y_fractions = 0
+        self.y_fractions = 0
         return True
 
     def clear_text(self):
