@@ -235,7 +235,8 @@ class Balloon:
         self.window.add(self.darea)
         self.darea.realize()
         self.window.realize()
-        self.window.get_window().set_background_pattern(None) # XXX
+        self.window.override_background_color(Gtk.StateFlags.NORMAL,
+                                              Gdk.RGBA(0, 0, 0, 0))
         self.set_skin()
         self.set_position()
         self.layout = None
