@@ -449,6 +449,8 @@ def read_surface_info(surface_dir):
                                      ninix.config.null_config())
     return surface, alias, tooltips
 
+re_alias = re.compile('^(sakura|kero|char[0-9]+)\.surface\.alias$')
+
 def read_surfaces_txt(surface_dir):
     config_list = []
     path = os.path.join(surface_dir, b'surfaces.txt')
@@ -494,7 +496,8 @@ def read_surfaces_txt(surface_dir):
                     if not opened:
                         logging.error(
                             'syntax error: unbalnced "}" in surfaces.txt.')
-                    if key in ['sakura.surface.alias', 'kero.surface.alias']:
+                    match = re_alias.match(key)
+                    if match:
                         alias_buffer.append(key)
                         alias_buffer.append('{')
 #                        alias_buffer.extend(buf)
@@ -694,7 +697,8 @@ def test():
                 if alias:
                     buf = []
                     for k, v in alias.items():
-                        if k in ['sakura.surface.alias', 'kero.surface.alias']:
+                        match = re_alias.match(k)
+                        if match:
                             print(''.join((k, ':')))
                             for alias_id, alias_list in v.items():
                                 print(alias_id, \
