@@ -267,7 +267,8 @@ class Skin:
         self.window.connect('delete_event', self.delete)
         self.window.connect('key_press_event', self.key_press)
         self.window.realize()
-        self.window.get_window().set_background_pattern(None) # XXX
+        self.window.override_background_color(Gtk.StateFlags.NORMAL,
+                                              Gdk.RGBA(0, 0, 0, 0))
         self.window.add_accel_group(self.accelgroup)
         self.darea = Gtk.DrawingArea()
         self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK|
@@ -630,7 +631,8 @@ class Katochan:
         self.window.add(self.darea)
         self.darea.realize()
         self.window.realize()
-        self.window.get_window().set_background_pattern(None) # XXX
+        self.window.override_background_color(Gtk.StateFlags.NORMAL,
+                                              Gdk.RGBA(0, 0, 0, 0))
         self.window.show()
         self.id = 0
         self.set_surface()
