@@ -408,6 +408,20 @@ class Sakura:
             surface_win = self.surface.get_window(side)
             balloon_win.set_transient_for(surface_win)
 
+    def update_balloon_offset(self, side, x_delta, y_delta):
+        if side >= self.char:
+            return
+        ox, oy = self.surface.window[side].balloon_offset # without scaling
+        direction = self.balloon.window[side].direction
+        sx, sy = self.get_surface_position(side)
+        if direction == 0: # left
+            nx = ox + x_delta
+        else:
+            w, h = self.surface.get_surface_size(side)
+            nx = ox - x_delta
+        ny = oy + y_delta
+        self.surface.set_balloon_offset(side, (nx, ny))
+
     def enqueue_script(self, event, script, sender, handle,
                        host, show_sstp_marker, use_translator,
                        db=None, request_handler=None):
@@ -511,7 +525,9 @@ class Sakura:
         self.balloon.set_position(side, base_x, base_y)
 
     def set_balloon_direction(self, side, direction):
-        self.balloon.set_direction(side, direction)
+        if side >= self.char:
+            return
+        self.balloon.window[side].direction = direction
 
     def get_balloon_size(self, side):
         result = self.balloon.get_balloon_size(side)
@@ -2047,6 +2063,14 @@ class Sakura:
                 self.__listening['OnOtherSurfaceChange'] = False
             else:
                 pass ## FIXME
+        elif args[0:2] == ['set', 'balloonoffset'] and argc > 3:
+            try:
+                x = int(args[2])
+                y = int(args[3])
+            except:
+                pass
+            else:
+                self.surface.set_balloon_offset(self.script_side, (x, y))
         elif args[0] == 'sound' and argc > 1:
             command = args[1]
             if self.audio_player is None:
