@@ -144,6 +144,10 @@ class Surface:
             if name == 'f12':
                 logging.info('reset surface position')
                 self.reset_position()
+            if name == 'f10':
+                logging.info('reset balloon offset')
+                for side in range(len(self.window)):
+                    self.set_balloon_offset(side, None)
         if name or keycode:
             self.request_parent(
                 'NOTIFY', 'notify_event', 'OnKeyPress', name, keycode,
@@ -405,12 +409,11 @@ class Surface:
             surface = elements[basename][0]
             if n == 0: # base surface
                 surface_list = [surface]
-            elif method == 'overlay':
-                surface_list.append((surface, x, y))
-            elif method == 'overlayfast': # XXX
-                surface_list.append((surface, x, y))
-            elif method == 'base':
-                surface_list.append((surface, x, y))
+            elif method in {'overlay', 'overlayfast',
+                            'interpolate', 'reduce', 'replace', 'asis'}:
+                surface_list.append((surface, x, y, method))
+            ##elif method == 'base':
+            ##    surface_list.append((surface, x, y))
             else:
                 error = 'unknown method for {0}: {1}'.format(key, method)
                 break
@@ -633,9 +636,17 @@ class Surface:
 
     def get_balloon_offset(self, side):
         if len(self.window) > side:
-            return self.window[side].get_balloon_offset()
+            x, y = self.window[side].balloon_offset
+            scale = self.window[side].scale
+            x = int(x * scale / 100)
+            y = int(y * scale / 100)
+            return x, y
         return 0, 0
 
+    def set_balloon_offset(self, side, offset):
+        if len(self.window) > side:
+            self.window[side].balloon_offset = offset
+
     def toggle_bind(self, args):
         side, bind_id = args
         self.window[side].toggle_bind(bind_id)
@@ -701,6 +712,7 @@ class SurfaceWindow:
         self.x_root = None
         self.y_root = None
         self.click_count = 0
+        self.__balloon_offset = None
         self.window.connect('leave_notify_event', self.window_leave_notify) # XXX
         self.window.connect('enter_notify_event', self.window_enter_notify) # XXX
         # create drawing area
@@ -728,7 +740,8 @@ class SurfaceWindow:
         self.darea.drag_dest_add_uri_targets()
         self.window.add(self.darea)
         self.window.realize()
-        self.window.get_window().set_background_pattern(None) # XXX
+        self.window.override_background_color(Gtk.StateFlags.NORMAL,
+                                              Gdk.RGBA(0, 0, 0, 0))
 
     def display_changed(self, screen):
         if self.side == 0:
@@ -768,6 +781,43 @@ class SurfaceWindow:
     def scale(self):
         return self.request_parent('GET', 'get_preference', 'surface_scale')
 
+    @property
+    def balloon_offset(self):
+        if self.__balloon_offset is None:
+            path, config = self.surface_info[''.join(('surface',
+                                                      self.surface_id))]
+            side = self.side
+            if side == 0:
+                name = 'sakura'
+                x = config.get_with_type(
+                    '{0}.balloon.offsetx'.format(name), int)
+                y = config.get_with_type(
+                    '{0}.balloon.offsety'.format(name), int)
+            elif side == 1:
+                name = 'kero'
+                x = config.get_with_type(
+                    '{0}.balloon.offsetx'.format(name), int)
+                y = config.get_with_type(
+                    '{0}.balloon.offsety'.format(name), int)
+            else:
+                name = 'char{0:d}'.format(side)
+                x, y = None, None # XXX
+            if x is None:
+                x = self.desc.get_with_type(
+                    '{0}.balloon.offsetx'.format(name), int, 0)
+            if y is None:
+                y = self.desc.get_with_type(
+                    '{0}.balloon.offsety'.format(name), int, 0)
+            self.__balloon_offset = x, y
+        else:
+            x, y = self.__balloon_offset
+        return x, y
+
+    @balloon_offset.setter
+    def balloon_offset(self, offset):
+        self.__balloon_offset = offset # (x, y)
+        self.request_parent('NOTIFY', 'reset_balloon_position')
+
     def drag_data_received(self, widget, context, x, y, data, info, time):
         filelist = []
         for uri in data.get_uris():
@@ -876,32 +926,56 @@ class SurfaceWindow:
             else:
                 raise RuntimeError('should not reach here')
 
-    def create_surface_from_file(self, surface_id):
+    def create_surface_from_file(self, surface_id, is_asis=False):
         assert surface_id in self.surfaces
-        use_pna = self.request_parent('GET', 'get_preference', 'use_pna')
+        if is_asis:
+            use_pna = False
+            is_pnr = False
+        else:
+            use_pna = self.request_parent('GET', 'get_preference', 'use_pna')
+            is_pnr = True
         try:
             surface = ninix.pix.create_surface_from_file(
-                self.surfaces[surface_id][0], use_pna=use_pna)
+                self.surfaces[surface_id][0], is_pnr=is_pnr, use_pna=use_pna)
         except:
             logging.debug('cannot load surface #{0}'.format(surface_id))
             return ninix.pix.create_blank_surface(100, 100)
-        cr = cairo.Context(surface)
-        for element, x, y in self.surfaces[surface_id][1:]:
+        for element, x, y, method in self.surfaces[surface_id][1:]:
             try:
+                if method == 'asis':
+                    is_pnr = False
+                    use_pna = False
+                else:
+                    is_pnr = True
+                    use_pna = self.request_parent(
+                        'GET', 'get_preference', 'use_pna')
                 overlay = ninix.pix.create_surface_from_file(
-                    element, use_pna=use_pna)
+                    element, is_pnr=is_pnr, use_pna=use_pna)
             except:
                 continue
+            cr = cairo.Context(surface)
+            op = {
+                'overlay':     cairo.OPERATOR_OVER,
+                'overlayfast': cairo.OPERATOR_ATOP,
+                'interpolate': cairo.OPERATOR_SATURATE,
+                'reduce':      cairo.OPERATOR_DEST_IN,
+                'replace':     cairo.OPERATOR_SOURCE,
+                'asis':        cairo.OPERATOR_OVER,
+                }[method]
+            cr.set_operator(op)
             cr.set_source_surface(overlay, x, y)
-            cr.mask_surface(overlay, x, y)
-        del cr
+            if method in {'overlay', 'overlayfast'}:
+                cr.mask_surface(overlay, x, y)
+            else:
+                cr.paint()
+            del cr
         return surface
 
-    def get_image_surface(self, surface_id):
+    def get_image_surface(self, surface_id, is_asis=False):
         if surface_id not in self.surfaces:
             logging.debug('cannot load surface #{0}'.format(surface_id))
             return ninix.pix.create_blank_surface(100, 100)
-        return self.create_surface_from_file(surface_id)
+        return self.create_surface_from_file(surface_id, is_asis=is_asis)
 
     def draw_region(self, cr):
         cr.save()
@@ -938,7 +1012,6 @@ class SurfaceWindow:
             surface_width = surface.get_width()
             surface_height = surface.get_height()
             done = []
-            cr = cairo.Context(surface)
             for actor in self.mayuna[surface_id]:
                 actor_id = actor.get_id()
                 if actor_id in self.bind and self.bind[actor_id][1] and \
@@ -946,14 +1019,17 @@ class SurfaceWindow:
                     done.append(actor_id)
                     for method, mayuna_id, dest_x, dest_y in self.iter_mayuna(surface_width, surface_height, actor, done):
                         mayuna_surface = self.get_image_surface(mayuna_id)
+                        cr = cairo.Context(surface)
                         if method in ['bind', 'add']:
                             cr.set_source_surface(mayuna_surface, dest_x, dest_y)
                             cr.mask_surface(mayuna_surface, dest_x, dest_y)
                         elif method == 'reduce':
-                            cr.mask_surface(mayuna_surface, dest_x, dest_y)
+                            cr.set_operator(cairo.OPERATOR_DEST_IN)
+                            cr.set_source_surface(mayuna_surface, dest_x, dest_y)
+                            cr.paint()
                         else:
                             raise RuntimeError('should not reach here')
-            del cr
+                        del cr
         else:
             surface = self.get_image_surface(surface_id)
         return surface
@@ -964,15 +1040,28 @@ class SurfaceWindow:
         # update collision areas
         self.collisions = self.region[self.seriko.base_id]
         # draw overlays
-        for surface_id, x, y in self.seriko.iter_overlays():
+        for surface_id, x, y, method in self.seriko.iter_overlays():
             try:
-                overlay_surface = self.get_image_surface(surface_id)
+                overlay_surface = self.get_image_surface(
+                    surface_id, is_asis=bool(method == 'asis'))
             except:
                 continue
             # overlay surface
             cr = cairo.Context(new_surface)
+            op = {
+                'overlay':     cairo.OPERATOR_OVER,
+                'overlayfast': cairo.OPERATOR_ATOP,
+                'interpolate': cairo.OPERATOR_SATURATE,
+                'reduce':      cairo.OPERATOR_DEST_IN,
+                'replace':     cairo.OPERATOR_SOURCE,
+                'asis':        cairo.OPERATOR_OVER,
+                }[method]
+            cr.set_operator(op)
             cr.set_source_surface(overlay_surface, x, y)
-            cr.mask_surface(overlay_surface, x, y)
+            if method in {'overlay', 'overlayfast'}:
+                cr.mask_surface(overlay_surface, x, y)
+            else:
+                cr.paint()
             del cr
         self.image_surface = new_surface
         self.darea.queue_draw()
@@ -994,8 +1083,8 @@ class SurfaceWindow:
     def remove_overlay(self, actor):
         self.seriko.remove_overlay(actor)
 
-    def add_overlay(self, actor, surface_id, x, y):
-        self.seriko.add_overlay(self, actor, surface_id, x, y)
+    def add_overlay(self, actor, surface_id, x, y, method):
+        self.seriko.add_overlay(self, actor, surface_id, x, y, method)
 
     def move_surface(self, xoffset, yoffset):
         x, y = self.get_position()
@@ -1005,29 +1094,6 @@ class SurfaceWindow:
             self.request_parent(
                 'NOTIFY', 'notify_observer', 'move surface', args) # animation
 
-    def get_balloon_offset(self):
-        path, config = self.surface_info[''.join(('surface', self.surface_id))]
-        side = self.side
-        if side == 0:
-            name = 'sakura'
-            x = config.get_with_type('{0}.balloon.offsetx'.format(name), int)
-            y = config.get_with_type('{0}.balloon.offsety'.format(name), int)
-        elif side == 1:
-            name = 'kero'
-            x = config.get_with_type('{0}.balloon.offsetx'.format(name), int)
-            y = config.get_with_type('{0}.balloon.offsety'.format(name), int)
-        else:
-            name = 'char{0:d}'.format(side)
-            x, y = None, None # XXX
-        if x is None:
-            x = self.desc.get_with_type('{0}.balloon.offsetx'.format(name), int, 0)
-        if y is None:
-            y = self.desc.get_with_type('{0}.balloon.offsety'.format(name), int, 0)
-        scale = self.scale
-        x = int(x * scale / 100)
-        y = int(y * scale / 100)
-        return x, y
-
     def get_collision_area(self, part):
         for p, x1, y1, x2, y2 in self.collisions: ## FIXME
             if p == part:
@@ -1090,7 +1156,10 @@ class SurfaceWindow:
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         direction = 0 if x > left + scrn_w // 2 else 1
         self.direction = direction
-        ox, oy = self.get_balloon_offset()
+        ox, oy = self.balloon_offset # without scaling
+        scale = self.scale
+        ox = int(ox * scale / 100)
+        oy = int(oy * scale / 100)
         if direction == 0: # left
             base_x = new_x + ox
         else:
